name = "seeq"
__version__ = '%d.%d.%d.%d.%d' % (int('$SQ(VERSION_MAJOR)'), int('$SQ(VERSION_MINOR)'), int('$SQ(VERSION_PATCH)'),
                                  int('$SQ(VERSION_MAJOR_PYPI)'), int('$SQ(VERSION_MINOR_PYPI)'))
